#ifndef Vector3DBase_H
#define Vector3DBase_H


#include "VectorTag.h"
#include "PV3DBase.h"

template <class T, class FrameTag> 
class Vector3DBase : public PV3DBase<T, VectorTag, FrameTag> {
public:

  typedef PV3DBase<T, VectorTag, FrameTag>    BaseClass;

  Vector3DBase() {}

  Vector3DBase(const T& x, const T& y, const T& z) : BaseClass(x, y, z) {}

  explicit Vector3DBase( const Basic3DVector<T>& v) : BaseClass(v) {}

  Vector3DBase unit() const { return Vector3DBase( basicVector().unit());}

  Vector3DBase& operator+= ( const Vector3DBase& v) {
    basicVector() += v.basicVector();
    return *this;
  } 
    
  Vector3DBase& operator-= ( const Vector3DBase& v) {
    basicVector() -= v.basicVector();
    return *this;
  } 
    
  Vector3DBase operator-() const { return Vector3DBase(-basicVector());}

  Vector3DBase& operator*= ( const T& t) { 
    basicVector() *= t;
    return *this;
  } 

  T dot( const Vector3DBase& v) const { 
    return basicVector().dot( v.basicVector());;
  }

  Vector3DBase cross( const  Vector3DBase& v) const {
    return Vector3DBase( basicVector().cross( v.basicVector()));
  }

};

template <class T, class FrameTag>
Vector3DBase<T,FrameTag> operator+( const Vector3DBase<T,FrameTag>& v1, 
				    const Vector3DBase<T,FrameTag>& v2) {
  return Vector3DBase<T,FrameTag>(v1) += v2;
}

template <class T, class FrameTag>
Vector3DBase<T,FrameTag> operator-( const Vector3DBase<T,FrameTag>& v1, 
				    const Vector3DBase<T,FrameTag>& v2) {
  return Vector3DBase<T,FrameTag>(v1) -= v2;
}

template <class T, class FrameTag>
T operator*( const Vector3DBase<T,FrameTag>& v1, 
	     const Vector3DBase<T,FrameTag>& v2) {
  return v1.dot(v2);
}

template <class T, class FrameTag>
Vector3DBase<T,FrameTag> operator*( const Vector3DBase<T,FrameTag>& v, 
				    float t) {
  return Vector3DBase<T,FrameTag>(v) *= t;
}

template <class T, class FrameTag>
Vector3DBase<T,FrameTag> operator*( float t, 
				    const Vector3DBase<T,FrameTag>& v) {
  return v*t;
}

template <class T, class FrameTag>
Vector3DBase<T,FrameTag> operator*( const Vector3DBase<T,FrameTag>& v, 
				    double t) {
  return Vector3DBase<T,FrameTag>(v) *= t;
}

template <class T, class FrameTag>
Vector3DBase<T,FrameTag> operator*( double t, 
				    const Vector3DBase<T,FrameTag>& v) {
  return v*t;
}

#endif // Vector3DBase_H
